

/*
 * Startup Code for ARM
 * This file is from the startup code version for AT91SAM7Sxx.
 *
 * Assembler file for IRQ
 * this file handles the very basic aspects of
 * interrupt handling. It acknowledges the interrupt,
 * saves registers and executes the corresponding handler.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */


#include "AT91SAM7S64.inc"
#include "swi_asm.h"

.text
	.func IRQ_Handler
	.global IRQ_Handler
IRQ_Handler:
	/* this handles the interrupt
	 * R13 contains the interrupt stack pointer,
	 * R14 contains the return address
	 * Interrupt service routines will be executed in User Mode
	 *
	 * stack needed in IRQ mode: r0,r1,lr,spsr = 16 bytes
	 */

	/* clear working register(s) */
	stmfd	sp!, {r0-r1,lr}

	/* store the SPSR for nested interrupts, also keep the return address */
	mrs	r0, SPSR
	stmfd	sp!, {r0}

	/* load the base address of AIC */
	ldr	r0, =AT91C_BASE_AIC

	/* acknowledge the interrupt,
	 * this is done by reading the interrupt handler vector
	 */
	ldr	r1, [r0, #AIC_IVR]

	/* switch to User Mode and enable Interrupts (nested Interrupts),
	 * keep FIQs as they are
	 */
	mrs	r0, CPSR		/* we must read the whole CPSR, but we can write back only the low part */
	and	r0, r0, #(ARM_F_BIT | ARM_MODE_User)
	orr	r0, r0, #ARM_MODE_User
	msr	CPSR_c, r0

	/* save registers in user mode stack */
	stmfd	sp!, {r2-r12,r14}
	/* call service routine */
	mov	lr, pc
	bx	r1
	/* restore registers that where saved in user mode stack */
	ldmia	sp!, {r2-r12,r14}
	/* switch back to IRQ mode and disable IRQ interrupts.
	 * FIQs should be kept as they are
	 */
	/* this must be done via SWI because it is not possible to reset
	 * the ARM Mode to IRQ when in User MODE */
	swi	SWI_IRQ

	/* tell the AIC that this interrupt has been handled */
	ldr	r0, =AT91C_BASE_AIC
	str	r1, [r0, #AIC_EOICR]

	/* restore environment r0,r1,lr,spsr*/
	ldmia	sp!, {r0}
	msr	SPSR_cxsf, r0
	ldmia	sp!, {r0-r1,lr}

	/* return from IRQ-Handler, adjust the value of return address */
	sub	lr, lr, #4
	movs	pc, lr	/* movS -> also restore SPSR to CPSR */

.equ	IRQ_Handler_Size, ( . - IRQ_Handler )

.global	IRQ_Handler_Size
.size IRQ_Handler, . - IRQ_Handler
.endfunc
