

/*
 * Startup Code for ARM
 * This file is from the startup code version for AT91SAM7Sxx.
 *
 * Header file for startup code
 * All startup-concerned defines are placed here. You must edit this
 * file to adapt it to your target configuration.
 * The clock options will be selected here.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */


#ifndef _STARTUP_H_
#define _STARTUP_H_

/* select the clock options here: */
/* if you want the Oscilator running, select this option: */
#define OSC_EN 1
/* select the oscilator frequency: */
#define F_OSC		18432000
/* the osc_startup is the time in ms to wait before the oscillator is
 * running. this should be looked up in the datasheet. it usually
 * is betwenn 1 and 14 ms. 0 is for SLOW clock operation */
#define OSC_STARTUP	1

/* if you want the pll to be enabled, select this one: */
#define PLL_EN 1
/* specifie the pll divisor and multiplyer. */
#define PLL_DIV		10
#define PLL_MUL		70
/* pll_count is the number of slow clock cycles before the
 * pll is taken to be locked */
#define PLL_COUNT	16

#define PLLCK	3
#define OSCCK	1
#define SLCK	0
/* put in here the source for the main clock */
#define MCK PLLCK

/* the prescaler for the main clock should be defined as follows:
 *  MCK_PRES contains the power of two that should be selected
 *    (0,1,2,3,4,5,6)
 *  MCK_PRESCALER will then contain the real divisor
 *    (1,2,4,8,16,32,64)
 */
#define MCK_PRES	(2)
#define MCK_PRESCALER	(1<<MCK_PRES)


/* -------------------------------------------------------------------
 * the rest of this file contains calculations used by startup_c.c
 * all user-specific changes should be made above!
 */

/* resolve dependancies */
#if ((PLL_EN == 1) && !defined(OSC_EN))
	/* the pll needs the OSC */
	#define OSC_EN 1
	#warning "The PLL needs the Oscillator as source. OSC activated."
#endif
#if ((MCK == PLLCK) && !defined(PLL_EN))
	#error "PLL set as clock source but not activated!"
#endif
#if ((MCK == OSCCK) && !defined(OSCCK_EN))
	#error "OSC set as clock source but not activated!"
#endif
#if ((defined(PLL_DIV) || defined(PLL_MUL) || defined(PLL_COUNT)) && !defined(PLL_EN))
	#warning "PLL parameters specified but not enabled."
#endif
#if ((defined(F_OSC) || defined(OSC_STARTUP)) && !defined(OSC_EN))
	#warning "OSC parameters specified but OSC not enabled."
#endif


/* calculate results */
/* F_CPU */
#if (MCK == PLLCK)
	#define F_CPU		((F_OSC)*((PLL_MUL)/((PLL_DIV)*(MCK_PRESCALER))))
#endif
#if (MCK == SLCK)
	#define F_CPU		(32000/MCK_PRESCALER)
#endif
#if (MCK == OSCCK)
	#define F_CPU		(F_OSC/MCK_PRESCALER)
#endif

/* PLL frequencies */
#if (PLL_EN == 1)
	#define F_PLL		((F_OSC)*(PLL_MUL)/(PLL_DIV))
	#define F_PLL_IN	(F_OSC/PLL_DIV)
#endif

/* Wait States */
#if (F_CPU > 30000000)
	#define FLASH_WAIT_STATES	1
#else
	#define FLASH_WAIT_STATES	0
#endif

/* control results for valid ranges */
#if (F_CPU > 55000000)
	#warning "Clock Frequency for CPU too high."
#endif
#if (F_PLL > 220000000)
	#warning "PLL Frequency too high."
#endif
#if (F_PLL < 80000000)
	#warning "PLL Frequency too low."
#endif
#if (F_PLL_IN < 1000000)
	#warning "PLL input Frequency too low."
#endif
#if (F_PLL_IN > 32000000)
	#warning "PLL input Frequency too high."
#endif

#endif
