

/*
 * Startup Code for ARM
 * This file is from the startup code version for AT91SAM7Sxx.
 *
 * C-Code for Initialization
 * this code intializes the .bss and .data sections. It also
 * selects the clock source options.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */


#include "AT91SAM7S64.h"
#define __inline extern inline
#include "lib_AT91SAM7S64.h"

#include "startup.h"

extern unsigned int _data_start, _data_load_start, _data_end, _bss_start, _bss_end;
	/* the handling of the addresses is a bit complicated.
	 * gcc assumes the values by the linker are addresses (eg 0x200000) to the value they should
	 * represent.
	 * thus a simple unsigned int *pointer = _data_start; will result in loading the value
	 * at the address (0x200000). this is not what is intended to be done, we want 0x200000 in
	 * pointer - pointer should point to 0x200000.
	 * this is achived by replacing _data_start with &_data_start.
	 */

/* AT91F_InitBSS
 * load .data from ROM to RAM
 * zero out .bss
 * The routine pretends, all addresses are 4-Byte-Aligned. Insert ". = ALIGN( 4 );" before
 * the definition of the PROVIDE( ... ); statement in the linker file to achieve this.
 * this procedure is executed in Supervisor mode
 */
void AT91F_InitBSS( void ) {
	unsigned int *dp, *sp;
	/* load the .data section into ram: */
	dp = &_data_start;
	sp = &_data_load_start;
	while (dp < &_data_end)
		*dp++ = *sp++;
	/* zero out the .bss section */
	dp = &_bss_start;
	while (dp < &_bss_end)
		*dp++ = 0;
}

/* AT91F_InitLowLevel
 * initialize the very basic things...
 * currently this is nothing, really nothing.
 * this procedure is executed in Supervisor mode
 */
void AT91F_InitLowLevel( void ) {
	/* enable Clock for PMC */
	AT91F_PMC_CfgPMC();

	/* disable the watchdog */
	AT91C_BASE_WDTC->WDTC_WDMR = AT91C_SYSC_WDDIS;

	/* start the embedded flash controller */
	AT91F_MC_CfgPMC();

	/* set the correct Flash Wait states */
	AT91C_BASE_MC->MC_FMR = ((FLASH_WAIT_STATES<<8) & AT91C_MC_FWS_3FWS) | (((F_CPU / 1000000)<<16) & AT91C_MC_FMCN);

	#if (OSC_EN == 1)
	/* startup the oscillator */
	/* startup time * slowclock / 8 = OSCount
	 * s[ms] * 4 = oscount
	 */
	AT91C_BASE_CKGR->CKGR_MOR = (OSC_STARTUP * 4) | AT91C_CKGR_MOSCEN;
	/* wait for oscillator ready */
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MOSCS));

	/* optional: read the frequency */
	/* if (AT91C_BASE_CKGR->CKGR_MCFR & AT91C_CKGR_MAINRDY)
		Frequency = (AT91C_BASE_CKGR->CKGR_MCFR & AT91C_CKGR_MAINF) * SCLK/16;
	*/

	#if (PLL_EN == 1)
	/* enable and configure the pll */
	AT91C_BASE_CKGR->CKGR_PLLR =
	#if ((PLL_MUL-1) * PLL_DIV > 155)
		(AT91C_CKGR_DIV & PLL_DIV) |
		(AT91C_CKGR_MUL & ((PLL_MUL-1)<<16)) |
		(AT91C_CKGR_PLLCOUNT & (PLL_COUNT<<8)) |
		AT91C_CKGR_OUT_2;
	#else
		(AT91C_CKGR_DIV & PLL_DIV) |
		(AT91C_CKGR_MUL & ((PLL_MUL-1)<<16)) |
		(AT91C_CKGR_PLLCOUNT & (PLL_COUNT<<8)) |
		AT91C_CKGR_OUT_0;
	#endif
	/* wait for the pll to lock */
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCK));
	#endif
	#endif

	#if (MCK == PLLCK)
	/* select the correct clock for the processor */
	AT91C_BASE_PMC->PMC_MCKR = AT91C_PMC_CSS_PLL_CLK | AT91C_PMC_PRES_CLK_4;
	#endif
	#if (MCK == OSCCK)
	#endif
	#if (MCK == SLCK)
	#endif

}


